% generates 4 systems of state-space matrices representing the double pendulum
% defined in the file doublep.m which must be in the path
% based on the mathematical model of Calum Iain Torrie et al. 1998
% function written by  K.A. Strain and Calum Iain Torrie 6/99
% version 0.5 - please do not edit without permission of the authors
% updated 14 Feb 2005 with corrected elements in the transverse/roll matrix from M Barton
% updated 29 June 2006 with corrected signs for the transverse/roll terms confirmed with M Barton
% updated 16 Oct 2006 - triple pendulum model reduced to a double pendulum by Chris Cueva

% NEW DESIGN
% VERSION HAS NO LEVER_ARM INPUT


% load in the pendulum parameters
doublep;


displaynormalmodefrequencies = 0; % if 1 display normal mode frequencies
                                  % if 0 do not display normal mode frequencies 
       
% general redefintion of variables to simplify notation
% and cope with different pendulum designs


     % The cantilevers each have a spring constant of kc1, etc. and the
     % wires each have a spring constant of kw1, etc.
     
% pend.ufc1 is the frequency the top mass will oscillate at if only supported by the cantilevers     
kc1	=  1/pend.nw1 * (2*pi*pend.ufc1)^2*pend.m1; % the factor of 1/pend.nw1 in front is because stage 1 has pend.nw1 cantilevers 	
kc2	=  1/pend.nw2 * (2*pi*pend.ufc2)^2*pend.m2; % the factor of 1/pend.nw2 in front is because stage 2 has pend.nw2 cantilevers
                                                % each wire is suspended from one cantilever
                                                
l1  =pend.l1;
l2  =pend.l2;

kw1	=pend.Y1*pi*pend.r1^2/l1;	
kw2	=pend.Y2*pi*pend.r2^2/l2;		

if (kc1==0)
	k1=kw1;				
else
	k1=kc1*kw1/(kc1+kw1);
end
if (kc2==0)
	k2=kw2;
else
	k2=kc2*kw2/(kc2+kw2);
end

k1 = k1*pend.nw1/2;
k2 = k2*pend.nw2/2;

if strcmp('single_pendulum',pend.pendulum_model)
        k2=0;
end
%********************************************************************

% allows choice of 2 wires to set separation to zero

s0=pend.su;
s1=pend.su;
s2=pend.si;
s3=pend.si;


d0=pend.d0;
d1=pend.d1;
d2=pend.d2;


%********************************************************************

m1 = pend.m1;
m2 = pend.m2;

I1x = pend.I1x;
I2x = pend.I2x;
I1y = pend.I1y;
I2y = pend.I2y;
I1z = pend.I1z;
I2z = pend.I2z;

n0  = pend.n0;
n1  = pend.n1;
n2  = pend.n2;
n3  = pend.n3;


%***********************************************************************************

% cosine and sine of the angle the wire makes with the vertical (z)

si1=(n1-n0)/l1;			%sin(omega1)
si2=(n3-n2)/l2;			%sin(omega2)
c1=(l1^2-(n1-n0)^2)^0.5/l1;	%cos(omega1)
c2=(l2^2-(n3-n2)^2)^0.5/l2;	%cos(omega2)











%**************************************************************************

%longitudinal and pitch equations of motion 
%errors in some terms corrected 6/99
% %(now including typo's found by Wensheng Hua)

k11	= -(m1+m2)*g*d0/I1y-2*k1*s0^2*c1^2/I1y-m2*g*d1/I1y-2*k2*s2^2*c2^2/I1y-m2*g*d1^2/I1y/l2/c2 ...
   -(m1+m2)*g*d0^2/I1y/l1/c1-(m1+m2)*g*s0^2*si1^2/I1y/l1/c1-m2*g*s2^2*si2^2/I1y/l2/c2;
k12	= -m2*g*d1/I1y/l2/c2+(m1+m2)*g*d0/I1y/l1/c1;
k13	= -m2*g*d1*d2/I1y/l2/c2+2*k2*s2^2*c2^2/I1y+m2*g*s2^2*si2^2/l2/c2/I1y;
k14	= +m2*g*d1/I1y/l2/c2;

k21	= +(m1+m2)*g*d0/m1/l1/c1-m2*g*d1/m1/l2/c2;
k22	= -(m1+m2)*g/m1/l1/c1-m2*g/m1/l2/c2;
k23	= -m2*g*d2/m1/l2/c2;
k24	= +m2*g/m1/l2/c2;
		

k31	= +2*k2*s2^2*c2^2/I2y-m2*g*d2*d1/I2y/l2/c2+m2*g*s2^2*si2^2/I2y/l2/c2;
k32	= -m2*g*d2/I2y/l2/c2 ;
k33	= -m2*g*d2/I2y-2*k2*s2^2*c2^2/I2y-m2*g*d2^2/I2y/l2/c2 ...
   -m2*g*s2^2*si2^2/l2/I2y/c2;
k34	= +m2*g*d2/I2y/l2/c2;


k41	= +m2*g*d1/m2/l2/c2;
k42	= +m2*g/m2/l2/c2;
k43	= +m2*g*d2/m2/l2/c2;
k44	= -m2*g/m2/l2/c2;


%state space matrices pitch and longitudinal
% masses in descending order (top = 1, intermediate = 2, mirror = 3)
% angle/displacement angle/displacement angle/displacement 
% unphysical damping used to make open loop phase plots readable
% set b = 0 for closed loop plots

% state [pitch1 long1 pitch2 long2 Vpitch1 Vlong1 Vpitch2 Vlong2]'
% input [longground Flong1 Flong1 Flong2 Torquepitch1 Torquepitch1 Torquepitch2]'
% output [long1 long2 pitch1 pitch2]'
A=[0	0	0	0		1	0	0	0	
   0	0	0	0		0	1	0	0				
   0	0	0	0		0	0	1	0	
   0	0	0	0		0	0	0	1		
    k11	k12	k13	k14		-bd	0	0	0	
    k21	k22	k23	k24		0	-bd	0	0	
    k31	k32	k33	k34 	0	0	-bd	0	
    k41	k42	k43	k44		0	0	0	-bd];

B=[0	0	0	0	 -(m1+m2)*g*d0/I1y/l1/c1 +(m1+m2)*g/m1/l1/c1   0	0	
   0	0	0	0		  0	1/m1	0	0	     
   0	0	0	0		  0	1/m1	0	0	     
   0    0   0   0        0  0	    0	1/m2	     
   0    0   0   0     1/I1y 0      0	0	    
   0    0   0   0     1/I1y 0      0	0	    
   0    0   0   0       0   0   1/I2y	0	    
       
]';

C=[0	1	0	0	0	0	0	0	
   0	0	0	1	0	0	0	0	
   1	0	0	0	0	0	0	0	
   0	0	1	0	0	0	0	0];

D=[0  0  0  0  0  0  0   
   0  0  0  0  0  0  0   
   0  0  0  0  0  0  0   
   0  0  0  0  0  0  0 ]; 

%%%%%%%%%%%%%%%%%%
lpe = ss(A,B,C,D);
%%%%%%%%%%%%%%%%%

% normal mode matrix
NMdoublepk = [k11 k12 k13 k14 
    k21 k22 k23 k24 
    k31 k32 k33 k34 
    k41 k42 k43 k44];
% calculation of the frequency
if displaynormalmodefrequencies, tiltlongfdoublep = sqrt(abs(eig(NMdoublepk)))/(2*pi), end

as = [0];
bs = [0];
cs = [0];
ds = [1];

% Removed CIT DEC 2001
%leverarm           = lever_pitch;

[a,b,c,d]   = append(as,bs,cs,ds,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);

%%%%%%%%%%%%%%%%%% Removed CIT DEC 2001
%[a,b,c,d]   = append(a,b,c,d,as,bs,cs,2*lever_pitch);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);  %NEW LINE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
[a,b,c,d]   = append(a,b,c,d,as,bs,cs,ds);
%[alp,blp,clp,dlp]   = series(a,b,c,d,A,B,C,D); changed Oct 17

%lpn = ss(alp,blp,clp,dlp); changed Oct 17


%**************************************************************************
% compute the A,B,C,D matrices and normal mode frequencies for rotational
% motion

AA	=(m1+m2)*g/l1/c1*(s0^2*c1^2+n0*n1)+2*k1*s0^2*(n1-n0)^2/l1^2;
BB	=m2*g/l2/c2*(s2^2*c2^2+n2*n3)+2*k2*s2^2*(n3-n2)^2/l2^2;
i11	=-AA/I1z-BB/I1z;
i12	=BB/I1z;
i21	=BB/I2z;
i22	=-BB/I2z;



% state space matrices
% state [sigma1 sigma2 vsigma1 vsigma2]'
% input [sigmaground Torquesigma1 Torquesigma1 Torquesigma2]'
% output [sigma1 sigma2]'
ar=[0	0	1	0	
   	0	0	0	1	
	i11	i12	-bd	0	 
	i21	i22	0	-bd];

br=[0	0  AA/I1z    0	
    0	0  1/I1z	 0	
    0	0  1/I1z	 0	
    0	0  0	    1/I2z]';

cr=[1	0	0	0		
	0	1	0	0];
dr=[0 0 0 0 
     0 0 0 0];

  %%%%%%%%%%%%%%%%%%%%%
 ye = ss(ar,br,cr,dr);
 %%%%%%%%%%%%%%%%%%%%%
 
 % normal mode matrix
NMdoublepr = [i11 i12 
    i21 i22];
% calculation of the frequency
if displaynormalmodefrequencies, rotationalfdoublep = sqrt(abs(eig(NMdoublepr)))/(2*pi), end
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Removed CIT DEC 2001 
%[acr,bcr,ccr,dcr]   = append(as,bs,cs,ds,as,bs,cs,lever_yaw*2);
[acr,bcr,ccr,dcr]   =  append(as,bs,cs,ds,as,bs,cs,ds); %NEW LINE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[acr,bcr,ccr,dcr]   =  append(acr,bcr,ccr,dcr,as,bs,cs,ds);
[acr,bcr,ccr,dcr]   =  append(acr,bcr,ccr,dcr,as,bs,cs,ds);
[acr,bcr,ccr,dcr]   =  append(acr,bcr,ccr,dcr,as,bs,cs,ds);
%[ay,by,cy,dy]       = series(acr,bcr,ccr,dcr,ar,br,cr,dr); changed Oct 17
%yn = ss(ay,by,cy,dy); changed Oct 17


    
    
% roll and transverse

j11	= -(m1+m2)*g*d0/I1x+(m1+m2)*g*n1*si1/I1x/c1-2*k1*n1^2/I1x-2*k1*n1*d0*si1/I1x/c1- ...
      m2*g*d1/I1x-m2*g*n2*si2/I1x/c2-2*k2*n2^2/I1x+2*k2*n2*d1*si2/I1x/c2+ ...
      (m1+m2)*g*d0/I1x*(n1*si1/l1-d0*c1/l1)-(m1+m2)*g*n1*si1/I1x/c1*(n1*si1/l1-d0*c1/l1)+ ...
      2*k1*l1*si1*n1/I1x*(n1*si1/l1-d0*c1/l1)+2*k1*l1*d0*si1^2/I1x/c1*(n1*si1/l1-d0*c1/l1)+ ...
      m2*g*d1/I1x*(-n2*si2/l2-d1*c2/l2)+m2*g*n2*si2/I1x/c2*(-n2*si2/l2-d1*c2/l2)- ...
      2*k2*l2*si2*n2/I1x*(-n2*si2/l2-d1*c2/l2)+2*k2*l2*d1*si2^2/I1x/c2*(-n2*si2/l2-d1*c2/l2);
   j12	= -m2*g*d1*c2/I1x/l2-m2*g*n2*si2/I1x/l2+2*k2*si2*n2*c2/I1x-2*k2*d1*si2^2/I1x+ ...
      (m1+m2)*g*d0*c1/I1x/l1-(m1+m2)*g*n1*si1/I1x/l1+2*k1*si1*n1*c1/I1x+2*k1*d0*si1^2/I1x;
   j13	= m2*g*d1/I1x*(n3*si2/l2-d2*c2/l2)+m2*g*n2*si2/I1x/c2*(n3*si2/l2-d2*c2/l2)- ...
      2*k2*l2*si2*n2/I1x*(n3*si2/l2-d2*c2/l2)+2*k2*l2*d1*si2^2/I1x/c2*(n3*si2/l2-d2*c2/l2)+ ...
      2*k2*n2*n3/I1x-2*k2*d1*si2*n3/I1x/c2;
   j14	= +m2*g*d1*c2/I1x/l2+m2*g*n2*si2/I1x/l2-2*k2*si2*n2*c2/I1x+2*k2*d1*si2^2/I1x;
   
   
   j21	= -(m1+m2)*g/m1*(n1*si1/l1-d0*c1/l1)-2*k1*l1*si1^2/m1/c1*(n1*si1/l1-d0*c1/l1)+ ...
      2*k1*n1*si1/m1/c1+m2*g/m1*(-n2*si2/l2-d1*c2/l2)+2*k2*l2*si2^2/m1/c2*(-n2*si2/l2-d1*c2/l2)+ ...
      2*k2*n2*si2/m1/c2;
   j22	= -(m1+m2)*g*c1/m1/l1-m2*g*c2/m1/l2-2*k1*si1^2/m1-2*k2*si2^2/m1;
   j23	= -2*k2*n3*si2/m1/c2+m2*g/m1*(n3*si2/l2-d2*c2/l2)+2*k2*l2*si2^2/m1/c2*(n3*si2/l2-d2*c2/l2);
   j24	= +m2*g*c2/m1/l2+2*k2*si2^2/m1;
   		
   
   j31	=+m2*g*d2/I2x*(-n2*si2/l2-d1*c2/l2)-m2*g*n3*si2/I2x/c2*(-n2*si2/l2-d1*c2/l2)+ ...
      2*k2*l2*si2*n3/I2x*(-n2*si2/l2-d1*c2/l2)+2*k2*l2*d2*si2^2/I2x/c2*(-n2*si2/l2-d1*c2/l2)+ ...
      2*k2*n2*n3/I2x+2*k2*d2*n2*si2/I2x/c2;
   j32	= -(+m2*g*d2*c2/I2x/l2-m2*g*n3*si2/I2x/l2+2*k2*si2*n3*c2/I2x+2*k2*d2*si2^2/I2x);
   j33	= +m2*g*d2/I2x*(n3*si2/l2-d2*c2/l2)-m2*g*n3*si2/I2x/c2*(n3*si2/l2-d2*c2/l2)+ ...
      2*k2*l2*si2*n3/I2x*(n3*si2/l2-d2*c2/l2)+2*k2*l2*d2*si2^2/I2x/c2*(n3*si2/l2-d2*c2/l2)- ...
      2*k2*n3*n3/I2x-2*k2*d2*n3*si2/I2x/c2-m2*g*d2/I2x+m2*g*n3*si2/I2x/c2;
   j34	= m2*g*d2*c2/I2x/l2-m2*g*n3*si2/I2x/l2+2*k2*si2*n3*c2/I2x+2*k2*d2*si2^2/I2x;
    
   j41	= +m2*g*si2*n2/m2/l2 +m2*g*c2*d1/m2/l2 +2*k2*si2^3*n2/m2/c2+2*k2*si2^2*d1/m2-2*k2*n2*si2/m2/c2;
   j42	= +m2*g*c2/m2/l2+2*k2*si2^2/m2;
   j43	= -m2*g*n3*si2/m2/l2+m2*g*d2*c2/m2/l2-2*k2*l2*si2^2/m2/c2*(n3*si2/l2-d2*c2/l2)+2*k2*n3*si2/m2/c2;
   j44	= -m2*g*c2/m2/l2-2*k2*si2^2/m2;
   
   
   
   
   % yyy and zzz from Mark Barton - see e-mail 7 Feb 2005
   yyy =  (2*d0*k1*n0^2)/(c1*I1x*l1)-(d0*g*(m1+m2)*n0^2)/(I1x*l1^2)-(2*d0*k1*n0^4)/(c1*I1x*...
l1^3)-(2*k1*n0*n1)/I1x-(2*d0*k1*n0*n1)/(c1*I1x*l1)+(d0*g*(m1+m2)*n0*n1)/(I1x*l1^2)...
+(6*d0*k1*n0^3*n1)/(c1*I1x*l1^3)+(2*k1*n0^3*n1)/(I1x*l1^2)-(g*(m1+m2)*n0^3*n1)/(...
c1*I1x*l1^3)-(6*d0*k1*n0^2*n1^2)/(c1*I1x*l1^3)-(4*k1*n0^2*n1^2)/(I1x*l1^2)+(2*...
g*(m1+m2)*n0^2*n1^2)/(c1*I1x*l1^3)+(2*d0*k1*n0*n1^3)/(c1*I1x*l1^3)+(2*k1*n0*n1^3)/...
(I1x*l1^2)-(g*(m1+m2)*n0*n1^3)/(c1*I1x*l1^3); 


zzz = (-2*c1*k1*n0^2)/(l1*m1)+(g*(m1+m2)*n0^2)/(l1^2*m1)+(2*c1*k1*n0*n1)/(l1*m1)-(g*(m1+m2)*...
n0*n1)/(l1^2*m1);

% coordinates follow plan for pitch and long
% state [roll1 trans1 roll2 trans2 Vroll1 Vtrans1 Vroll2 Vtrans2]'
% in [transground Ftrans1 Ftrans1 Torqueroll1 Torqueroll1 rollground]'
% out [trans1 trans2 roll1 roll2]'
A=[0	0	0	0	1	0	0	0	
   0	0	0	0	0	1	0	0				
   0	0	0	0	0	0	1	0	
   0	0	0	0	0	0	0	1		
   j11	j12	j13	j14	-bd	0	0	0	
   j21	j22	j23	j24	0	-bd	0	0	
   j31	j32	j33	j34	0	0	-bd	0	
   j41	j42	j43	j44	0	0	0	-bd];

% signs of yyy and zzz corrected on June 2006 - Chris Cueva
B=[0	0	0	0	 -(m1+m2)*g*d0*c1/I1x/l1+(m1+m2)*g*n1*si1/I1x/l1-2*k1*si1*n1*c1/I1x-2*k1*d0*si1^2/I1x +(m1+m2)*g*c1/m1/l1+2*k1*si1^2/m1   0	0
   0	0	0	0	     	0	1/m1	0	0	    
   0	0	0	0	     	0	1/m1	0	0	    
   0  0  0  0         -yyy       -zzz	0	0	
   0  0  0  0         1/I1x       0	0	0	
   0  0  0  0         1/I1x       0	0	0	
]';


C=[0	1	0	0	0	0	0	0	
   0	0	0	1	0   0	0	0	
   1	0	0	0	0	0	0	0	
   0	0	1	0	0	0	0	0];

D=[0  0  0  0 0 0
    0  0  0  0 0 0
    0  0  0  0 0 0
   0  0  0  0 0 0];

     %%%%%%%%%%%%%%%%
   rte = ss(A,B,C,D);
   %%%%%%%%%%%%%%%%%%
   
   
[ac,bc,cc,dc]   = append(as,bs,cs,ds,as,bs,cs,ds);
[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Removed CIT DEC 2001
%[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,3*lever_roll);
[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);  %NEW LINE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);
[ac,bc,cc,dc]   = append(ac,bc,cc,dc,as,bs,cs,ds);
[art,brt,crt,drt]   = series(ac,bc,cc,dc,A,B,C,D);
rtn = ss(art,brt,crt,drt);

% normal mode matrix
NMdoublepj = [j11 j12 j13 j14
    j21 j22 j23 j24
    j31 j32 j33 j34 
    j41 j42 j43 j44 ];
% calculation of the frequency
if displaynormalmodefrequencies, transverseandrollfdoublep = sqrt(abs(eig(NMdoublepj)))/(2*pi), end

%**************************************************************************
% compute the A,B,C,D matrices and normal mode frequencies for vertical
% motion

h1=2*k1*c1^2+(m1+m2)*g*si1^2/l1/c1;
h2=2*k2*c2^2+m2*g*si2^2/l2/c2;
h11 = - h1/m1 - h2 /m1;
h12 = h2/m1  ;
h21 = h2 / m2;
h22 = - h2 / m2 ;



% state space matrices
% state [z1 z2 vz1 vz2]'
% input [zground Forcez1 Forcez1]'
% output [z1 z2]'

av= [ 	0	0 	1 	0		
        0	0	0	1				
	    h11	h12	-bd	0		
	    h21	h22	 0	-bd];

bv= [ 0	0  	h1 /m1  0   	 	
      0	0	1 / m1	0	   
      0	0	1 / m1	0	   
     ]';

cv= [1	0	0	0		
	  0	1	0	0];
    
dv= [0 0 0 
      0 0 0];

    %%%%%%%%%%%%%
  ve = ss(av,bv,cv,dv);
  %%%%%%%%%%%%%
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Removed CIT DEC 2001
%[acv,bcv,ccv,dcv]   = append(as,bs,cs,ds,as,bs,cs,3);
[acv,bcv,ccv,dcv]   = append(as,bs,cs,ds,as,bs,cs,ds); %NEW LINE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[acv,bcv,ccv,dcv]   = append(acv,bcv,ccv,dcv,as,bs,cs,ds);
[av,bv,cv,dv]   = series(acv,bcv,ccv,dcv,av,bv,cv,dv);
vn = ss(av,bv,cv,dv);
% normal mode matrix
NMdoublepv = [h11 h12
    h21 h22];
% calculation of the frequency
if displaynormalmodefrequencies, verticalfdoublep = sqrt(abs(eig(NMdoublepv)))/(2*pi), end